//+------------------------------------------------------------------+
//|                                               TSSv1_2.mq4        |
//|                      Copyright  2007,Pip2,Isto,jam,Oleg77
//|                       pips4life, Chandra
//|                                        http://www.metaquotes.net |
//+------------------------------------------------------------------+
/* This indicator is based on T3MA, now called OT3MA set at period 4 and
Volume Factor 1.5. and places arrows at peaks/\ and valleys\/ and Alarms at
Buy and Sell points.   Started by Pip2 and enhanced by jam, isto and Oleg77 */
//
// Modified by ACS. Last Modified: 14-MAY-07.
// Useful comments from the FF discussion thread (by Pip2):
// Common sense in trading still holds true. Don't sell in an up trend, and don't buy in a downtrend. 
// Always buy or sell on pullbacks in the direction of the trend. In a ranging market, caution is the rule as always.
// I used the triple smoothed MA called T3MA with a period of 4 and a volume factor of 1.5 instead of 0.7. This reduces the lag quite a bit.
// I use T3MA(28,0.7) to determine the Main trend on H4 charts.
// To recap, TF's H1,H4 and maybe D1, use a trend indicator, 
// also Stochastic(10,4,4) helps as a confirming indicator.
// you can also use an OT3MA(15,0.80) as a visual filter for trends.
// You're absolutely right but ranging markets vary widely and some good ranges could yield many pips while others might yield losses. 
// If the range is narrow and choppy I would sit on my hands, especially the index finger that does all the clicking.
// But if a range is wide, say 50 to 90 pips, bouncing from lower Boll Band to Upper Boll band, you will see arrows above 
// and below the bands and those could be very profitable. So the nature of the range is important in determining if it could be profitable.
// The longer TF's require larger stops. I always use a trailing stop based on the rough average pips (H-L) of bars near bar 0. 
// Stop losses are something else. I don't trust them, just like insurance companies, they're a necessary evil. 
// You can use Parabolic SAR to place stops because that's what that indicator was designed for from my research on indicators.
// In a long trend, P. SAR could act as a trailing stop because it reduces the stop as the trend progresses.
// Cheers, Pip2 (Snobol1)
// ACS: Can be used with 15M chart (EURJPYm checked) with SAR(0.02,0.2); even 5M chart seems reasonable. (Also see MACDS-Dir, ADX_DI-Dir in 5M).
// ACS: Best way seems to be to use two timeframes; trade only in the direction of higher timeframe signal;
//      (1H & 15M+SAR seems good; 1D & 1H or 4H & 1H also should be good). 1H signal against SAR should be considered as weak, but OK to use.
//      Better to wait for the lower timeframe SAR to agree. Also better to use 2-pip breakout from signal bar to enter?
// Check: Pivot levels, RLB/TLB/FLIR, Candles, Trend channel, 5,6-EMA-COX also.
// TODO: Make new version. Add additional calculation to get hourly signal and show only when both are in agreement.

#include <stderror.mqh>
//#include <stdlib.mq4>    // this was removed because using this blocked the Input tab. The error description was changed to just error code.
#property indicator_chart_window
#property indicator_buffers 2
#property indicator_color1 Yellow
#property indicator_color2 Magenta
#property indicator_width1 2
#property indicator_width2 2

extern int    period      = 4;
extern double volfac      = 1.5;
extern int Strength = 1;
extern int MaximumBars=200;
extern bool   Alarmset    = false;
extern bool   alarm2Mail  = true;
extern int MinTimePercent = 35;
int    arrowSize   = 2;
bool ShowComment=false;
string str1="Enter P0 and P1 for slope";
int    P0          = 0;
int    P1          = 1;
string T3MA_Name   = "OT3MAV3_1";

double        upbuffer[];
double        dnbuffer[];
double        ax[];
int           sep=1,ba,sa; //b=Buy thresh s=Sell thresh.
static int    alarmBar, alarmBar1;  // keeps track of the bar where alarm has been fired
bool          _up, _dn;
//+------------------------------------------------------------------+
//| Custom indicator initialization function                         |
//+------------------------------------------------------------------+
int init() {
   if(arrowSize < 1) arrowSize = 1;
   if(arrowSize > 5) arrowSize = 5;
   if(Period() < 15 ) alarm2Mail=false;  // don't want to get emails for smaller time frame charts.
//---- 1 additional buffer is used for TSS.
   IndicatorBuffers(3);

//---- drawing settings
   IndicatorDigits(Digits+2);
   SetIndexStyle(0,DRAW_ARROW,3,arrowSize);
   SetIndexArrow(0,233);//This is buy signal
   SetIndexStyle(1,DRAW_ARROW,3,arrowSize);
   SetIndexArrow(1,234);
   SetIndexStyle(2, DRAW_NONE);

//---- 3 indicator buffers mapping
   SetIndexBuffer(0, upbuffer);
   SetIndexBuffer(1, dnbuffer);
   SetIndexBuffer(2, ax);

//---- name for DataWindow and indicator subwindow label
   IndicatorShortName("TSSv1_2");
   SetIndexLabel(0,"BUY ");
   SetIndexLabel(1,"SELL ");
   SetIndexLabel(2,NULL);

//---- if not values
   SetIndexEmptyValue(0,0.0);
   SetIndexEmptyValue(1,0.0);
   SetIndexEmptyValue(2,0.0);
//---- alart has been never fired yet
   alarmBar = -1; alarmBar1 = -1;
return(0); }
//+------------------------------------------------------------------+
//| Custom indicator deinitialization function                       |
//+------------------------------------------------------------------+
int deinit()   {     return(0);   }
//+------------------------------------------------------------------+
//| Custom indicator iteration function                              |
//+------------------------------------------------------------------+
int start(){ 
   int i, counted_bars=IndicatorCounted();
   if(counted_bars<0)  { return(-1);  }
//---- last counted bar will be recounted
   if(counted_bars>0) { counted_bars--; }
//  if(counted_bars == 0) return(0); // nothing to do - there is no ticks // ACS: without this commented, it does not work.
   int limit=Bars-counted_bars; if (limit > MaximumBars) limit=MaximumBars;

   for(i=0; i<limit; i++)  ax[i]=iCustom(NULL,0,T3MA_Name,period,volfac,0,i);  //---- TSS counted in the 1st additional buffer

   for(i=0; i<limit; i++) {
      if(ax[i]-ax[i+1]<Strength*Point && ax[i+1]-ax[i+2]>Strength*Point)     { _dn=true; } else { _dn=false; }  // ACS changed zero to Strength.
      if((ax[i]-ax[i+1])>Strength*Point && (ax[i+1]-ax[i+2])<Strength*Point) { _up=true; } else { _up=false; }
//---- sell signal
      if( _dn && dnbuffer[i] == 0 )  {  dnbuffer[i] = High[i]+(sep*Point);  }    // store value only of is not has been stored already
      if( !_dn && dnbuffer[i] != 0 && i==0 )  { dnbuffer[i] = 0;  } // removes false/changed signal for current bar.
//---- buy signal
      if( _up && upbuffer[i] == 0)   {  upbuffer[i] = Low[i]-(sep*Point);   }
      if( !_up && upbuffer[i] != 0 && i==0 )  { upbuffer[i] = 0;  }// removes false/changed signal for current bar.
  } // end for
   if(counted_bars == 0) return(0); // nothing to do - there is no ticks // this was above, commented earlier; I un-commented and using here!
   if (alarm2Mail) SendMsg();  if (Alarmset) doAlarm();  if (ShowComment) setComment();
return(0); }
//+------------------------------------------------------------------+
//| Custom function - send a mail message for each generated signal  |
//+------------------------------------------------------------------+
void SendMsg() {
   string subj, msg, msg1,Dir="";
   int check, Timeleft; double TimeLeft1, SARdiff;
   if (upbuffer[0] == 0 && dnbuffer[0] == 0) return;  // no signal to mail
   if ( alarmBar1 >= Bars ) return;  // mail has been sent already
   Timeleft= Period()*60 - (TimeCurrent() - Time[0]); // Print("Minutes left in the current bar = ",Timeleft/60);
   TimeLeft1=Timeleft/60.0/Period();
   if (TimeLeft1 > 1.0-MinTimePercent/100.0) return;  // if it is an early signal, ignore. wait for the time limit to pass for that bar.
   if (upbuffer[0] != 0)  { subj="Buy signal for "+Symbol()+" ("+Period()+"M)"; msg="TSSv1_2  -->> BUY"; alarmBar1 = Bars; }
   if (dnbuffer[0] != 0)  { subj="Sell signal for "+Symbol()+" ("+Period()+"M)"; msg="TSSv1_2  -->> SELL"; alarmBar1 = Bars; }
   SARdiff=(Ask+Bid)/2 - iSAR(NULL,0,0.02,0.2,0); if (SARdiff > 0) Dir ="BUY"; if (SARdiff < 0) Dir = "SELL";
   msg1 = StringConcatenate(TimeToStr(TimeLocal(),TIME_DATE)," ",TimeToStr(TimeLocal(),TIME_MINUTES)," ",Symbol()," (",Period(),"M Chart)");
   msg1 = msg1+"\n"+msg+ "; SAR -->> = "+Dir+";  [Minutes left in the current bar = "+Timeleft/60+"]"+
            "\nTodays OHLC prices are: "+
            DoubleToStr(iOpen(Symbol(),PERIOD_D1,0),Digits)+", "+
            DoubleToStr(iHigh(Symbol(),PERIOD_D1,0),Digits)+", "+
            DoubleToStr(iLow(Symbol(),PERIOD_D1,0),Digits)+", "+
            DoubleToStr(iClose(Symbol(),PERIOD_D1,0),Digits);
   SendMail(subj, msg1);
   check=GetLastError();
      Print("subject: ", subj, "; body: ", msg1);
//  if(check!=ERR_NO_ERROR) Print("Cannot send message, error: ", ErrorDescription(check)); // this function needed stdlib.mq4
   if (check!=ERR_NO_ERROR) Print("Cannot send message! The error code was: ",check);
return; }
//+------------------------------------------------------------------+
//        Alarm section                                              +
//+------------------------------------------------------------------+
// if buffer has been set and if this is another bar than the bar alarm has been fired for the last time
void doAlarm() {
   if (alarmBar >= Bars )  return;  // alert has been fired already
   if (upbuffer[0] == 0 && dnbuffer[0] == 0)  return;  // no signal to alarm
Print("UpBuffer: " , upbuffer[0], "; DownBuffer: ", dnbuffer[0]);
   if (upbuffer[0] != 0)  { Alert (Symbol()," ",Period(),"TSSv1_2  >> BUY "); alarmBar = Bars; }
   if (dnbuffer[0] != 0)  { Alert (Symbol()," ",Period(),"TSSv1_2  >> SELL "); alarmBar = Bars; }
return; }
//+---------------------------------------------------------+
//        Comment section                                   +
//+---------------------------------------------------------+
void setComment() {
   double bx0=iMA(NULL,PERIOD_H1,50,0,MODE_LWMA,PRICE_CLOSE,P0);//PERIOD_H1
   double bx1=iMA(NULL,PERIOD_H1,50,0,MODE_LWMA,PRICE_CLOSE,P1);
   double slope=(bx0-bx1)/Point;

   double cx0=iMA(NULL,PERIOD_D1,50,0,MODE_LWMA,PRICE_CLOSE,P0);//PERIOD_H1
   double cx1=iMA(NULL,PERIOD_D1,50,0,MODE_LWMA,PRICE_CLOSE,P1);
   double slope2=(cx0-cx1)/Point;

   Comment("SLOPE PIPS PER PERIOD_H1 = ",slope, "\n\nSLOPE PIPS PER PERIOD_D1 = ",slope2," AT P0=",P0," & P1=",P1);
return; }
//+------------------------------------------------------------------+

